#!/usr/bin/python
# -*- coding: utf-8 -*-
import requests
import re
import pickle
import ciso8601
import time
import json

try:
    # python 3
    from urllib.parse import urlencode
except ImportError:
    # python 2
    from urllib import urlencode


class FabricClient(object):
    __user_agent = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36'}

    __fabric_projects_ids = {'your_app_name1': 'your_app_id1', 'your_app_name2': 'your_app_id2'}

    def __init__(self):
        pass

    @staticmethod
    def save_cookies(requests_cookiejar, filename):
        with open(filename, 'wb') as f:
            pickle.dump(requests_cookiejar, f)

    @staticmethod
    def load_cookies(filename):
        with open(filename, 'rb') as f:
            return pickle.load(f)

    def __get_project_id(self, application_id):
        try:
            return self.__fabric_projects_ids[application_id]
        except KeyError:
            print "Error: Fabric project for application id ~" + application_id + "~ is not specified."
            raise

    def __get_auth_cookies(self):
        r = requests.post('https://fabric.io/api/v2/session', headers=
        {'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
         'Referer': 'https://www.fabric.io/login',
         'origin': 'https://www.fabric.io',
         'Accept': 'application/json, text/javascript, */*; q=0.01',
         'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36',
         'X-CSRF-Token': self.__request_login_form()},
                          data=urlencode({'password': 'your_password', "email": 'your@mail.com'}),
                          cookies=self.load_cookies("COOKIE"))
        self.save_cookies(r.cookies, "COOKIE")
        # my_buffer = StringIO()
        # c = pycurl.Curl()
        # c.setopt(c.URL, 'https://fabric.io/api/v2/session')
        # c.setopt(c.POSTFIELDS, postfields)
        # c.setopt(c.COOKIEJAR, self.__cookie_file)
        # c.setopt(c.COOKIEFILE, self.__cookie_file)
        # c.setopt(c.WRITEDATA, my_buffer)
        # c.setopt(c.HTTPHEADER, ['Content-Type:application/x-www-form-urlencoded; charset=UTF-8',
        #            'Referer:https://www.fabric.io/login',
        #            'Origin:https://www.fabric.io',
        #            'Accept:application/json, text/javascript, */*; q=0.01',
        #            'User-Agent:Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36',
        #            'X-CSRF-Token:' + token])
        # c.setopt(c.HEADER, 1)
        # c.perform()
        # c.close()
        return "Cookies saved"

    def __request_login_form(self):
        # my_buffer = StringIO()
        # c = pycurl.Curl()
        # c.setopt(c.URL, 'https://www.fabric.io/login')
        # c.setopt(c.COOKIEJAR, self.__cookie_file)
        # c.setopt(c.COOKIEFILE, self.__cookie_file)
        # c.setopt(c.HTTPHEADER, [
        #     'user-agent: Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36'])
        # c.setopt(c.WRITEDATA, my_buffer)
        # c.perform()
        # c.close()
        r = requests.get('https://www.fabric.io/login', headers=self.__user_agent)
        self.save_cookies(r.cookies, "COOKIE")
        m = re.search('<meta content="(?P<token>[^"]+)" name="csrf-token" />', r.text)
        if m.group() == "None":
            print "Failed to locate the CSRF token."
            raise
        else:
            return m.group('token')

    def query(self, url):
        self.__get_auth_cookies()
        response = requests.get(url, headers=self.__user_agent, cookies=self.load_cookies("COOKIE"))
        if response.status_code != 200:
            print "Unexpected response code " + str(response.status_code)
            raise
        return response.content

    # Работает по-другому: сам создает файл csv с контентом
    def query_csv(self, url):
        temp_string = self.query(url)
        temp_string = ''.join(temp_string).replace('"', '')

        if re.search('.*event_type.*', url):
            filename = url[url.index('event_type=') + 11:] + '.csv'
        else:
            filename = 'output.csv'
        with open(filename, 'wb') as f:
            f.write(temp_string)
        return "done"

    # Поэтому данный метод ничего не возвращает
    def get_stats(self, start_time, end_time, app_id, stats_type):
        start_time = ciso8601.parse_datetime(start_time)
        end_time = ciso8601.parse_datetime(end_time)
        self.query_csv('https://fabric.io/api/v3/projects/%s/answers_csv/%s_csv?start=%s&end=%s&build=all' % (
            self.__get_project_id(app_id), stats_type, str(int(time.mktime(start_time.timetuple()))),
            str(int(time.mktime(end_time.timetuple())))))
        return

    # И этот тоже
    def get_event_stats(self, start_time, end_time, app_id, stats_type, event_name):
        start_time = ciso8601.parse_datetime(start_time)
        end_time = ciso8601.parse_datetime(end_time)

        self.query_csv('https://fabric.io/api/v3/projects/%s/answers_csv/ce_%s_csv?start=%s&end=%s&event_type=%s' % (
            self.__get_project_id(app_id), stats_type, str(int(time.mktime(start_time.timetuple()))),
            str(int(time.mktime(end_time.timetuple()))), event_name))
        return

    # Возвращает список ивентов
    def get_event_list(self, start_time, end_time, app_id):
        start_time = ciso8601.parse_datetime(start_time)
        end_time = ciso8601.parse_datetime(end_time)
        self.__get_auth_cookies()
        # X-CRASHLYTICS-DEVELOPER-TOKEN нигде не нашел в запросах, но он не меняется
        # поэтому статикой его сюда прописал
        r = requests.get(
            'https://fabric.io/api/v2/organizations/YOUR_APP_ID/apps/%s/growth_analytics/event_types.json?start=%s&end=%s&build=all' % (
                self.__get_project_id(app_id), str(int(time.mktime(start_time.timetuple()))),
                str(int(time.mktime(end_time.timetuple())))),
            headers=
            {'Content-Type': 'application/json; charset=utf-8',
             'Accept': 'application/json, text/javascript, */*; q=0.01',
             'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36',
             'X-CRASHLYTICS-DEVELOPER-TOKEN': 'TOKEN'},
            cookies=self.load_cookies("COOKIE"))
        # Разбор json-а: ищем custom_events, далее внутри него бежим по всем ивентам и ищем event_type
        # затем регексом смотрим содержит ли слово НУЖНОЕ_СЛОВО
        datas = json.loads(r.content)
        temp_list = []
        for item in datas:
            if item == "custom_events":
                for event in datas[item]:
                    for node in event:
                        if node == "event_type":
                            if re.search('.*SOME_WORD.*', event[node]) or re.search('.*some_word.*', event[node]):
                                temp_list.append(event[node])
        return temp_list
