#!/usr/bin/python
# -*- coding: utf-8 -*-
import requests
import pickle
import json


class iTunesClient(object):
    __itunes_projects_ids = {'your_app_name1': '123456789', 'your_app_name2': '987654321'}

    def __init__(self):
        pass

    @staticmethod
    def save_cookies(requests_cookiejar, filename):
        with open(filename, 'wb') as f:
            pickle.dump(requests_cookiejar, f)

    @staticmethod
    def load_cookies(filename):
        with open(filename, 'rb') as f:
            return pickle.load(f)

    def __get_project_id(self, application_id):
        try:
            return self.__itunes_projects_ids[application_id]
        except KeyError:
            print "Error: iTunes project for application id ~" + application_id + "~ is not specified."
            raise

    def get_woa_cookies(self):
        self.get_auth_cookies()
        r = requests.get('https://itunesconnect.apple.com/WebObjects/iTunesConnect.woa',
                         headers={'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
                                  'Referer': 'https://itunesconnect.apple.com/itc/static/login',
                                  'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36'},
                         cookies=self.load_cookies('COOKIE'), allow_redirects=False)
        my_jar = self.load_cookies('COOKIE')
        for i in my_jar:
            if i.name == 'myacinfo':
                r.cookies.set_cookie(i)
        self.save_cookies(r.cookies, 'COOKIE')
        return "Cookies saved"

    def get_auth_cookies(self):
        self.request_login_cookies()
        r = requests.post('https://idmsa.apple.com/appleauth/auth/signin', headers={
            'Referer': 'https://idmsa.apple.com/appleauth/auth/signin?widgetKey=YOUR_WIDGET_KEY',
            'Content-Type': 'application/json', 'X-Apple-Widget-Key': 'YOUR_WIDGET_KEY',
            'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36'},
                          data=json.dumps({'password': 'your_password', 'accountName': 'your@mail.com',
                                           'rememberMe': 'false'}), cookies=self.load_cookies("COOKIE"))
        self.save_cookies(r.cookies, "COOKIE")
        return "Cookies saved"

    def request_login_cookies(self):
        r = requests.get('https://idmsa.apple.com/appleauth/auth/signin?widgetKey=YOUR_WIDGET_KEY',
                         headers={'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
                                  'Referer': 'https://itunesconnect.apple.com/itc/static/login',
                                  'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36'})
        self.save_cookies(r.cookies, "COOKIE")
        return "saved"

    def get_latest_version(self, app_id):
        self.get_woa_cookies()
        r = requests.get(
            'https://itunesconnect.apple.com/WebObjects/iTunesConnect.woa/ra/apps/%s/reviews/summary?platform=ios' % self.__get_project_id(
                app_id), headers={'Accept': 'application/json, text/plain, */*',
                                  'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36'},
            cookies=self.load_cookies('COOKIE'))
        datas = json.loads(r.content)
        temp_list = []
        max_version = 0
        for node in datas:
            if node == "data":
                for item in datas[node]:
                    if item == 'versions':
                        for versions in datas[node][item]:
                            temp_list.append(int(versions))
                        max_version = max(temp_list)
        return max_version

    def get_ratings(self, app_id, version):
        self.get_woa_cookies()
        r = requests.get(
            'https://itunesconnect.apple.com/WebObjects/iTunesConnect.woa/ra/apps/%s/reviews/summary?platform=ios&storefront=US&versionId=%s' % (
                self.__get_project_id(
                    app_id), version), headers={'Accept': 'application/json, text/plain, */*',
                                                'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36'},
            cookies=self.load_cookies('COOKIE'))
        datas = json.loads(r.content)
        temp_dict = {}
        for node in datas:
            if node == "data":
                for item in datas[node]:
                    if item == 'ratings':
                        temp_dict['Total'] = self.make_dict(datas[node][item])
                    elif item == 'storeFronts':
                        for country in datas[node][item]:
                            if country['countryCode'] == 'US':
                                temp_dict['US'] = self.make_dict(country['ratings'])
                            elif country['countryCode'] == 'GB':
                                temp_dict['GB'] = self.make_dict(country['ratings'])
                            elif country['countryCode'] == 'CA':
                                temp_dict['CA'] = self.make_dict(country['ratings'])
                            elif country['countryCode'] == 'DE':
                                temp_dict['DE'] = self.make_dict(country['ratings'])
                            elif country['countryCode'] == 'AU':
                                temp_dict['AU'] = self.make_dict(country['ratings'])
        return temp_dict

    def get_units(self, app_id):
        self.get_woa_cookies()
        print self.load_cookies("COOKIE")
        r = requests.post('https://analytics.itunes.apple.com/analytics/api/v1/data/app/detail/measures',
                          headers={'Accept': 'application/json, text/plain, */*',
                                   'Referer': 'https://analytics.itunes.apple.com/',
                                   'Content-Type': 'application/json;charset=UTF-8',
                                   'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36'},
                          data=json.dumps(
                              {'adamId': ['YOUR_ID'], 'frequency': 'DAY', 'measures': ['units'],
                               'startTime': '2017-09-28T00:00:00Z', 'endTime': '2017-10-04T00:00:00Z'}),
                          cookies=self.load_cookies("COOKIE"))
        print r.status_code
        print r.headers
        print r.content
        return True

    @staticmethod
    def make_dict(my_dict):
        temp_dict = {}
        for i in my_dict:
            if i == 'ratingFourCount':
                temp_dict[4] = my_dict[i]
            elif i == 'ratingThreeCount':
                temp_dict[3] = my_dict[i]
            elif i == 'ratingOneCount':
                temp_dict[1] = my_dict[i]
            elif i == 'ratingFiveCount':
                temp_dict[5] = my_dict[i]
            elif i == 'ratingTwoCount':
                temp_dict[2] = my_dict[i]
        return temp_dict
